;spi_single_byte.a51  Test SPI RAM by sending and reading back a single byte
; Nov 10, renamed July 15

; rev path 3/16
	$NOSYMBOLS		; keeps listing short
	$INCLUDE (C:\SiLabs\MCU\INC\c8051f410.inc) 
	$INCLUDE (C:\SiLabs\MCU\INC\VECTORS320.INC)	; Tom's vectors definition file

	STACKBOT EQU 80h				
; PORT USE:
; PORT2: keypad
; PORT1: display

;SPI:
; P0.2 - SPI SCK    (digital output, push-pull)
; P0.3 - SPI MISO   (digital input, open-drain)
; P0.6 - SPI MOSI   (digital output, push-pull)
; P0.7 - SPI NSS    (digital output, push-pull)

; register use: R7 command; R6,R5 address, R4 data to write, R3 data read from RAM

	WRITE_COM EQU 02h		; write command
	READ_COM EQU 03h		; read command
	WRITE_STAT EQU 01h	; write status register
	READ_STAT EQU 05h		; read status register
	MODE_BYTE	EQU 01h		; storage mode
	MODE_PAGE EQU 81h
	MODE_SEQU EQU 41h

	CS EQU P0.6					; for manual control of CS*, rather than relying on NSS, which jumps up midway in tfr

	KEYPAD EQU P2
	DISPLAY EQU P1

	ORG 0h
	LJMP STARTUP

	ORG 040h
				
STARTUP: MOV SP, #STACKBOT-1

	acall USUAL_SETUP
	acall Init_Device

	; send mode select, to choose single-byte operation

	mov R4, #MODE_BYTE			; set up single-byte transfer
	mov R7, #WRITE_STAT			; set up command to set status
	acall COMMAND_WRITE
	acall SMALL_DELAY

; now see if I can read back what's just written to status register
	mov R7, #READ_STAT
	acall COMMAND_READ	
	mov DISPLAY, R3			; show what comes back from SPI: should be 01h
	acall SMALL_DELAY


	; now write a byte
WRITE_ONE:	mov R7, #WRITE_COM  ; addr sender will use this to send appropriate command
	mov R6, #24h							; high byte of address
	mov R5, #45h							; ...low byte

	mov R4, KEYPAD					; get data to write, from keypad

	acall ADDR_SENDER
		acall SMALL_DELAY


	acall SEND_DATA					; this finishes the transmission, with a byte of data
	setb NSSMD0
	acall SMALL_DELAY

; ...and read it back
	mov R7, #READ_COM			; now get the value back from RAM
	acall ADDR_SENDER
	acall GET_DATA
	acall SMALL_DELAY

	mov DISPLAY, R3				; show what RAM delivers

	sjmp $								; this to test single transfer
;	sjmp WRITE_ONE				; ...this to do continual transfers
	


;--Command_Only-- takes command type in R7, and control value (mode) in R4
COMMAND_WRITE: 
				clr NSSMD0						; assert CS* of peripheral 
				mov SPI0DAT, R7			; send command byte
				jnb SPIF, $						; hang here till data-transferred flag is asserted
			  clr SPIF						; clear data-transferred flag
	acall SMALL_DELAY

				mov SPI0DAT, R4			; send status (mode) control
				jnb SPIF, $						; hang here till data-transferred flag is asserted
			setb NSSMD0					; disassert CS*
				ret

COMMAND_READ:   clr NSSMD0
				mov SPI0DAT, R7			; send command
				jnb SPIF, $						; hang here till data-transferred flag is asserted
			  clr SPIF						; clear data-transferred flag
				acall SMALL_DELAY

				mov SPI0DAT, #0FFh			; send dummy value
				jnb SPIF, $						; hang here till data-transferred flag is asserted
			  clr SPIF						; clear data-transferred flag

; the next patch, waiting for termination of BUSY, is necessary
COMMAND_BUSY:			mov A, SPI0CFG				; elaborate way to branch on SPIBSY
			  jb acc.7, COMMAND_BUSY		; hang while it's high
				mov R3, SPI0DAT			; get what has come in
			  setb NSSMD0					; disassert CS*
				acall SMALL_DELAY

				ret

;---ADDR SENDER : R7 holds command, R6, R5 hold 16-bit address, This serves for data WRITE and READ
;  this does not complete any transfer: only assertes CS*, then sends command and 16-bit address
ADDR_SENDER:
			clr NSSMD0						; assert CS* of peripheral 
			mov SPI0DAT, R7			; send command byte
			jnb SPIF, $						; hang here till data-transferred flag is asserted
			clr SPIF	; clear data-transferred flag
	acall SMALL_DELAY

			mov SPI0DAT, R6					; send high byte of address
			jnb SPIF, $						; hang here till data-transferred flag is asserted
			clr SPIF						; clear data-transferred flag
	acall SMALL_DELAY

			mov SPI0DAT, R5					; send low byte of address
			jnb SPIF, $						; hang here till data-transferred flag is asserted
			clr SPIF						; clear data-transferred flag
	acall SMALL_DELAY
			ret


SEND_DATA:		mov A, KEYPAD					; get value to send
			mov SPI0DAT, A
			jnb SPIF, $						; hang here till data-transferred flag is asserted
			clr SPIF						; clear data-transferred flag

	acall SMALL_DELAY
			ret


GET_DATA:	mov SPI0DAT, #0FFh			; send dummy data
				jnb SPIF, $						; hang here till data-transferred flag is asserted
			  clr SPIF						; clear data-transferred flag

; the next patch, waiting for termination of BUSY, is necessary
DATA_BUSY:			mov A, SPI0CFG				; elaborate way to branch on SPIBSY
			  jb acc.7, DATA_BUSY		; hang while it's high
				mov R3, SPI0DAT			; get what has come in

			setb NSSMD0					; disassert CS*
				ret



; --MODE: R4 holds mode: byte,page or sequential
MODE_SET: clr NSSMD0						; assert CS* of peripheral 
				mov SPI0DAT, #01h			; send command byte
				jnb SPIF, $						; hang here till data-transfer flag is asserted

			  clr SPIF								; clear data-transferred flag
			  mov SPI0DAT, R4					; send mode
			  jnb SPIF, $							; hang here till data-transferred flag is asserted
			  clr SPIF								;	clear data-transferred flag

				setb NSSMD0
			  ret	
	
; small delay (to make SPI bursts easier to see on scope) --------
SMALL_DELAY: push acc
;			mov a, #10h
			mov a, #2h
			djnz acc, $
			pop acc
			ret



;------- INITIALIZATIONS

USUAL_SETUP: anl   PCA0MD, #NOT(040h)        ; Disable the WDT
				; Configure the Oscillator
;            orl   OSCICN, #04h         ; sysclk = 24.5 Mhz / 8
			mov OSCICN, #087h				; sysclk = 24.5 MHz  

	    ret
;------------------------------------
;-  Generated Initialization File  --
;------------------------------------

; these initializations are more than what's required for single-byte write/read test

; Peripheral specific initialization functions,
; Called from the Init_Device label
SPI_Init:
    mov  SPI0CFG,   #040h
;    mov  SPI0CN,    #009h
		mov  SPI0CN,    #00Dh     ; spi: set NSSMD0 (= CS*) high at outset
    ret

ADC_Init:
    mov  ADC0MX,    #001h
    mov  ADC0CN,    #084h
    ret

DAC_Init:
    mov  IDA0CN,    #0F2h
    mov  IDA1CN,    #0F6h
    ret

Voltage_Reference_Init:
    mov  REF0CN,    #013h
    ret

Port_IO_Init:
    ; P0.0  -  Skipped,     Open-Drain, Analog
    ; P0.1  -  Skipped,     Open-Drain, Analog
    ; P0.2  -  SCK  (SPI0), Push-Pull,  Digital
    ; P0.3  -  MISO (SPI0), Open-Drain, Digital
    ; P0.4  -  Skipped,     Push-Pull,  Digital
    ; P0.5  -  Skipped,     Open-Drain, Digital
    ; P0.6  -  MOSI (SPI0), Push-Pull,  Digital
    ; P0.7  -  NSS  (SPI0), Push-Pull,  Digital

    ; P1.0  -  Unassigned,  Open-Drain, Digital
    ; P1.1  -  Unassigned,  Open-Drain, Digital
    ; P1.2  -  Skipped,     Open-Drain, Analog
    ; P1.3  -  Unassigned,  Open-Drain, Digital
    ; P1.4  -  Unassigned,  Open-Drain, Digital
    ; P1.5  -  Unassigned,  Open-Drain, Digital
    ; P1.6  -  Unassigned,  Open-Drain, Digital
    ; P1.7  -  Unassigned,  Open-Drain, Digital

    ; P2.0  -  Unassigned,  Open-Drain, Digital
    ; P2.1  -  Unassigned,  Open-Drain, Digital
    ; P2.2  -  Unassigned,  Open-Drain, Digital
    ; P2.3  -  Unassigned,  Open-Drain, Digital
    ; P2.4  -  Unassigned,  Open-Drain, Digital
    ; P2.5  -  Unassigned,  Open-Drain, Digital
    ; P2.6  -  Unassigned,  Open-Drain, Digital
    ; P2.7  -  Unassigned,  Open-Drain, Digital

    mov  P0MDIN,    #0FCh
    mov  P1MDIN,    #0FBh
    mov  P0MDOUT,   #0D4h
    mov  P0SKIP,    #033h
    mov  P1SKIP,    #004h
    mov  XBR0,      #002h
    mov  XBR1,      #040h
    ret

Interrupts_Init:
    mov  IT01CF,    #005h  ; assigns INT0 to P0.5
    mov  IE,        #081h  ; enable global and EX0 interrupts
    ret

; Initialization function for device,
; Call Init_Device from your main program
Init_Device:
    lcall SPI_Init
    lcall ADC_Init
    lcall DAC_Init
    lcall Voltage_Reference_Init
    lcall Port_IO_Init
    lcall Interrupts_Init
    ret

end
